% ----------------------------------------------------------------------- %
% Example of use of the funcion MOPSO.m, which performs a Multi-Objective %
% Particle Swarm Optimization (MOPSO), based on Coello2004.               %
% ----------------------------------------------------------------------- %
%   Author:  Victor Martinez Cagigal                                      %
%   Date:    15/03/2017                                                   %
%   E-mail:  vicmarcag (dot) gmail (dot) com                              %
% ----------------------------------------------------------------------- %
%   References:                                                           %
%       Coello, C. A. C., Pulido, G. T., & Lechuga, M. S. (2004). Handling%
%       multiple objectives with particle swarm optimization. IEEE Tran-  %
%       sactions on evolutionary computation, 8(3), 256-279.              %
% ----------------------------------------------------------------------- %
clear all; clc;

% Multi-objective function
nfun = 5;
if(nfun==1)     % Schaffer N1
    MultiObj.fun = @(x) [x(:).^2, (x(:)-2).^2];
    MultiObj.nVar = 1;
    MultiObj.var_min = -5;
    MultiObj.var_max = 5;
elseif(nfun==2) % Kursawe 
    MultiObj.fun = @(x) [-10.*(exp(-0.2.*sqrt(x(:,1).^2+x(:,2).^2)) + exp(-0.2.*sqrt(x(:,2).^2+x(:,3).^2))), ...
                         sum(abs(x).^0.8 + 5.*sin(x.^3),2)];
    MultiObj.nVar = 3;
    MultiObj.var_min = -5.*ones(1,MultiObj.nVar);
    MultiObj.var_max = 5.*ones(1,MultiObj.nVar);
elseif(nfun==3) % Zitzler-Deb-Thiele's N3
    g  = @(x) 1+(9/29).*sum(x(:,2:end),2);
    h  = @(x) 1 - sqrt(x(:,1)./g(x)) - (x(:,1)./g(x)).*sin(10.*pi.*x(:,1));
    MultiObj.fun = @(x) [x(:,1), g(x).*h(x)];
    MultiObj.nVar = 30;
    MultiObj.var_min = 0.*ones(1,MultiObj.nVar);
    MultiObj.var_max = 1.*ones(1,MultiObj.nVar);
elseif(nfun==4) % Poloni's two-objective
    A1 = 0.5*sin(1)-2*cos(1)+sin(2)-1.5*cos(2);
    A2 = 1.5*sin(1)-cos(1)+2*sin(2)-0.5*cos(2);
    B1 = @(x,y) 0.5.*sin(x)-2.*cos(x)+sin(y)-1.5.*cos(y);
    B2 = @(x,y) 1.5.*sin(x)-cos(x)+2.*sin(y)-0.5.*cos(y);
    f1 = @(x,y) 1+(A1-B1(x,y)).^2+(A2-B2(x,y)).^2;
    f2 = @(x,y) (x+3).^2+(y+1).^2;
    MultiObj.fun = @(x) [f1(x(:,1),x(:,2)), f2(x(:,1),x(:,2))];
    MultiObj.nVar = 2;
    MultiObj.var_min = -pi.*ones(1,MultiObj.nVar);
    MultiObj.var_max = pi.*ones(1,MultiObj.nVar);
elseif(nfun==5) % Viennet
    f1 = @(x,y) 0.5.*(x.^2+y.^2)+sin(x.^2+y.^2);
    f2 = @(x,y) (1/8).*(3.*x-2.*y+4).^2 + (1/27).*(x-y+1).^2 +15;
    f3 = @(x,y) (1./(x.^2+y.^2+1))-1.1.*exp(-(x.^2+y.^2));
    MultiObj.fun = @(x) [f1(x(:,1),x(:,2)), f2(x(:,1),x(:,2)), f3(x(:,1),x(:,2))];
    MultiObj.nVar = 2;
    MultiObj.var_min = [-3, -10];
    MultiObj.var_max = [10, 3];
end

% Parameters
params.Np = 200;        % Population size
params.Nr = 200;        % Repository size
params.maxgen = 200;    % Maximum number of generations
params.W = 0.4;         % Inertia weight
params.C1 = 2;          % Individual confidence factor
params.C2 = 2;          % Swarm confidence factor
params.ngrid = 20;      % Number of grids in each dimension
params.maxvel = 5;      % Maxmium vel in percentage

% MOPSO
MOPSO(params,MultiObj,figure(1));



    